"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClaimPersistHandler = void 0;
const types_1 = require("../../../onchain-certificate/types");
const onchain_certificate_service_1 = require("../../../onchain-certificate/onchain-certificate.service");
const Certificate_events_1 = require("../../events/Certificate.events");
const offchain_certificate_service_1 = require("../../offchain-certificate.service");
const common_1 = require("@nestjs/common");
const lodash_1 = require("lodash");
const batch_configuration_1 = require("../strategies/batch/batch.configuration");
let ClaimPersistHandler = class ClaimPersistHandler {
    constructor(certificateService, offchainCertificateService, batchConfiguration) {
        this.certificateService = certificateService;
        this.offchainCertificateService = offchainCertificateService;
        this.batchConfiguration = batchConfiguration;
    }
    canHandle(event) {
        return event.type === Certificate_events_1.CertificateEventType.Claimed;
    }
    async handle(event) {
        const claimedEvent = event;
        try {
            await this.certificateService.claim(claimedEvent.payload);
            await this.offchainCertificateService.claimPersisted(claimedEvent.internalCertificateId, { persistedEventId: event.id });
            return { success: true };
        }
        catch (e) {
            await this.offchainCertificateService.persistError(claimedEvent.internalCertificateId, {
                errorMessage: `${e.message}`,
                internalCertificateId: event.internalCertificateId,
                type: Certificate_events_1.CertificateEventType.ClaimPersisted,
                persistedEventId: event.id
            });
            return { success: false };
        }
    }
    async handleBatch(events) {
        const eventsBlocks = lodash_1.chunk(events, this.batchConfiguration.claimBatchSize);
        const failedCertificateIds = [];
        for (const eventsBlock of eventsBlocks) {
            const result = await this.synchronizeBatchBlock(eventsBlock);
            failedCertificateIds.push(...result.failedCertificateIds);
        }
        return { failedCertificateIds };
    }
    async synchronizeBatchBlock(events) {
        const claimedEvents = events;
        try {
            await this.certificateService.batchClaim(claimedEvents.map((event) => event.payload));
            const promises = events.map(async (event) => {
                await this.offchainCertificateService.claimPersisted(event.internalCertificateId, {
                    persistedEventId: event.id
                });
            });
            await Promise.all(promises);
            return { failedCertificateIds: [] };
        }
        catch (e) {
            const promises = events.map(async (event) => {
                await this.offchainCertificateService.persistError(event.internalCertificateId, {
                    errorMessage: `${e.message}`,
                    internalCertificateId: event.internalCertificateId,
                    type: Certificate_events_1.CertificateEventType.ClaimPersisted,
                    persistedEventId: event.id
                });
                return event.internalCertificateId;
            });
            await Promise.all(promises);
            return { failedCertificateIds: events.map((e) => e.internalCertificateId) };
        }
    }
};
ClaimPersistHandler = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(types_1.ONCHAIN_CERTIFICATE_SERVICE_TOKEN)),
    __param(2, common_1.Inject(batch_configuration_1.BATCH_CONFIGURATION_TOKEN)),
    __metadata("design:paramtypes", [onchain_certificate_service_1.OnChainCertificateService,
        offchain_certificate_service_1.OffChainCertificateService, Object])
], ClaimPersistHandler);
exports.ClaimPersistHandler = ClaimPersistHandler;
//# sourceMappingURL=claim-persist.handler.js.map